#define	STDIO_DEBUG_SERIAL	SADR
#define	STDIO_DEBUG_BAUD		57600

// Add this line to add carriage returns ('\r') to each newline
// char ('\n') when sending printf output to a serial port.

#define	STDIO_DEBUG_ADDCR

#define	TRW_LOCAL_ADDRESS 0x21,0x43,0x65,0x87,0xA9
#define	TRW_DATA_PORT	PFDR
#define	TRW_DATA_SHADOW	PFDRShadow
#define	TRW_DATA_DIR	PFDDR
#define	TRW_DATA_DSHADOW	PFDDRShadow
#define	TRW_SCK_PORT	PFDR
#define	TRW_SCK_SHADOW	PFDRShadow
#define	TRW_CS_PORT		PFDR
#define	TRW_CS_SHADOW	PFDRShadow
#define	TRW_CE_PORT		PFDR
#define	TRW_CE_SHADOW	PFDRShadow
#define	TRW_DR_PORT		PCDR
#define	TRW_DATA_BIT  	5
#define	TRW_SCK_BIT   	4
#define	TRW_CS_BIT		6
#define	TRW_CE_BIT		7
#define	TRW_DR_BIT		1

#use TRW-24g.lib


#define	TRW_REMOTE2_ADDRESS	0X13,0X25,0X46,0X79,0X8A
const static unsigned char raddr2[]={TRW_REMOTE2_ADDRESS};


//----------------------------------------------------------------------------------------
void calc_sht11(float *p_humidity ,float *p_temperature)
//----------------------------------------------------------------------------------------
// calculates temperature [C] and humidity [%RH]
// input :  humi [Ticks] (12 bit)
//          temp [Ticks] (14 bit)
// output:  humi [%RH]
//          temp [C]
{ const static float C1=-4.0;              // for 12 Bit
  const static float C2=+0.0405;           // for 12 Bit
  const static float C3=-0.0000028;        // for 12 Bit
  const static float T1=+0.01;             // for 14 Bit @ 5V
  const static float T2=+0.00008;           // for 14 Bit @ 5V

  float rh;             				// rh:      Humidity [Ticks] 12 Bit
  float t;					            // t:       Temperature [Ticks] 14 Bit
  float rh_lin;                     // rh_lin:  Humidity linear
  float rh_true;                    // rh_true: Temperature compensated humidity
  float t_C;                        // t_C   :  Temperature [C]

  rh=*p_humidity;
  t=*p_temperature;
  t_C=t*0.01 - 39.63;                  //calc. temperature from ticks to [C]
  rh_lin=C3*rh*rh + C2*rh + C1;     //calc. humidity from ticks to [%RH]
  rh_true=(t_C-25)*(T1+T2*rh)+rh_lin;   //calc. temperature compensated humidity [%RH]
  if(rh_true>100)rh_true=100;       //cut if the value is outside of
  if(rh_true<0.1)rh_true=0.1;       //the physical possible range

  *p_temperature=t_C;               //return temperature [C]
  *p_humidity=rh_true;              //return humidity[%RH]
}

//--------------------------------------------------------------------
float calc_dewpoint(float h,float t)
//--------------------------------------------------------------------
// calculates dew point
// input:   humidity [%RH], temperature [C]
// output:  dew point [C]
{ float logEx,dew_point;
  logEx=0.66077+7.5*t/(237.3+t)+(log10(h)-2);
  dew_point = (logEx - 0.66077)*237.3/(0.66077+7.5-logEx);
  return dew_point;
}


void main()

{
unsigned int num;
int i;
unsigned char msg[TRW_DATA_LEN+1];
float humi_val,temp_val;
float dew_point;


	BitWrPortI (PDDDR,&PDDDRShadow,1,2);
   BitWrPortI (PDDR,&PDDRShadow,0,2 );					// PD2=0
	WrPortI (PFDDR,&PFDDRShadow,0xD0 );

	for(i=0; i<10; i++)
		for(num=0; num<32767; num++);
   trw_init();

  num=0;
  while(1){
	costate {
     	printf("Sent poll\n");
		wfd i=trw_poll(raddr2,msg,100);
      if(i>0){
     		humi_val=(float)(*(int *)(&msg[3]));               //converts integer to float
      	temp_val=(float)(*(int *)(&msg[1]));               //converts integer to float
	      calc_sht11(&humi_val,&temp_val);            //calculate humidity, temperature
   	   dew_point=calc_dewpoint(humi_val,temp_val); //calculate dew point
         msg[5]=0;
         printf("Got: %0x %04x %04x\n",(int) msg[0],*(int *)(&msg[1]),*(int *)(&msg[3]));
	      printf("\ttemperature: %5.1fC, humidity: %5.1f%%, dew point: %5.1fC\n",temp_val,humi_val,dew_point);
	      waitfor(DelayMs(1000));
		}
      num++;
	}
  }
}